"""
Модуль itmo-routespd позволяет строить индивидуальные образовательные маршруты
для помощи абитуриентам в выборе существующих
направлений подготовки, факультетов и рабочих программ дисциплин.

Vladislav Pivosh [https://github.com/RUGyron]
pivosh098@gmail.com
"""

__author__ = 'Vladislav Pivosh'

__version__ = '1.0'

from .BaseRouter import *
from .Data import *
from .DetailedRouter import *
